﻿// -----------------------------------------------------------------------
//  <last-editor>黄深</last-editor>
//  <last-date>2023-01-31 17:00</last-date>
// -----------------------------------------------------------------------

using System;
using System.Runtime.CompilerServices;
using KingMetalTemplateProject.Infrastructure.Database.Options;
using KingMetal.Infrastructures.Common.Utilities;
using LinqToDB.AspNet;
using LinqToDB.AspNet.Logging;
using LinqToDB.Configuration;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.DependencyInjection.Extensions;
using Microsoft.Extensions.Hosting;

namespace KingMetalTemplateProject.Infrastructure.Database.Extensions;

public static class ServiceCollectionExtensions
{
    #region 获取数据库连接对象

    /// <summary>
    ///     获取数据库连接对象
    /// </summary>
    /// <param name="service"> </param>
    /// <returns> </returns>
    public static DataBaseConnection GetDataConnection(this IServiceProvider service)
    {
        return service.GetService<DataBaseConnection>();
    }

    #endregion 获取数据库连接对象

    #region 添加数据库

    /// <summary>
    ///     添加数据库
    /// </summary>
    /// <param name="services"> </param>
    /// <param name="config">   配置委托 </param>
    /// <returns> </returns>
    [MethodImpl(MethodImplOptions.AggressiveInlining)]
    public static IServiceCollection AddDataConnection(this IServiceCollection services,
        Action<DataBaseOptions> config)
    {
        var dataBaseOptions = new DataBaseOptions();
        config(dataBaseOptions);

        if (string.IsNullOrEmpty(dataBaseOptions.ConnectionString))
            throw new ArgumentException("ConnectionString Can Not Be Null");

        services.AddLinqToDBContext<DataBaseConnection>((provider, options) =>
        {
            options.UsePostgreSQL(dataBaseOptions.ConnectionString);

            var environment = provider.GetService<IHostEnvironment>();
            if (!environment.IsProduction())
                options.UseDefaultLogging(provider);

        }, ServiceLifetime.Singleton);

        services.Replace(new ServiceDescriptor(typeof(DataBaseConnection), provider =>
        {
            var options = provider.GetService<LinqToDBConnectionOptions<DataBaseConnection>>();
            return new DataBaseConnection(options);
        }, ServiceLifetime.Transient));

        return services;
    }

    /// <summary>
    ///     添加数据库
    /// </summary>
    /// <param name="services">      </param>
    /// <param name="configuration"> 配置对象 </param>
    /// <returns> </returns>
    public static IServiceCollection AddDataConnection(this IServiceCollection services,
        IConfiguration configuration)
    {
        return services.AddDataConnection(m =>
        {
            var cfg = ConfigurationHelper.GetConfiguration(configuration, nameof(DataBaseOptions));
            m.ConnectionString = cfg.GetValue<string>("ConnectionString");
        });
    }

    #endregion 添加数据库
}