﻿// -----------------------------------------------------------------------
//  <last-editor>黄深</last-editor>
//  <last-date>2023-01-31 17:14</last-date>
// -----------------------------------------------------------------------

using System.IO;
using System.Threading.Tasks;
using Hs.Infrastructure.Common.Options;
using KingMetalTemplateProject.Infrastructure.Database.Extensions;
using Hs.Infrastructure.HostExtensions.Extensions;
using KingMetalTemplateProject.Infrastructure.Shared.Constants;
using KingMetal.Domains.UniqueValueService.Grains;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Orleans;
using Orleans.Hosting;
using Orleans.Streams;

internal class Program
{
    #region 主程序入口

    /// <summary>
    ///     主程序入口
    /// </summary>
    /// <returns> </returns>
    public static Task Main(string[] args)
    {
        return Host.CreateDefaultBuilder(args)
            .UseContentRoot(Directory.GetCurrentDirectory())
            .ConfigureAppConfiguration(builder => builder.AddUserSecrets<Program>())
            .ConfigureServices((context, services) =>
            {
                services.Configure<HuangsOptions>(context.Configuration.GetSection(nameof(HuangsOptions)));
                services.AddSingleton(context.Configuration);
                services.AddDataConnection(context.Configuration);
            })
            .ConfigurationKingMetalDefault(siloBuilder =>
            {
                siloBuilder.AddSimpleMessageStreamProvider(AppConstants.StreamProviderName, options =>
                    {
                        options.FireAndForgetDelivery = true;
                        options.PubSubType = StreamPubSubType.ExplicitGrainBasedOnly;
                    })
                    .AddMemoryGrainStorage(AppConstants.MemoryGrainStorageName)
                    .ConfigureApplicationParts(parts =>
                    {
                        parts.AddApplicationPart(typeof(UniqueValueServiceGrain).Assembly).WithReferences();
                        //TODO: Add Business Grain
                    });
            })
            .Build()
            .RunAsync();
    }

    #endregion 主程序入口
}